---
layout: page
title: "Documentation"
lang: fr
---

Vous trouverez ici des manuels, tutoriels et références pour apprendre
Ruby. Une bonne partie de ces ressources est en langue anglaise. Sachez
qu’il existe plusieurs livres en français. Toutefois, la pratique de
l’anglais est recommandée car il s’agit *de facto* de la langue
dominante en informatique. Bon courage !
{: .summary}

### Installer Ruby

À moins que vous ne vouliez essayer Ruby dans un navigateur (voir les liens
plus bas) vous avez besoin d'avoir Ruby installé sur votre ordinateur.
Vous pouvez vérifier si Ruby est déjà disponible en ouvrant une invite de
commande et en tapant

{% highlight sh %}
ruby -v
{% endhighlight %}

Ceci devrait afficher des informations sur la version de ruby qui est installée.
Dans le cas contraire, référez-vous à la [page d’installation](installation/)
pour les nombreuses façons d'obtenir Ruby.

### Débuter avec Ruby

#### Ressources en français

[Ruby en vingt minutes](/fr/documentation/quickstart/)
: Ce tutoriel simple d’accès, **en français**, couvre les fondamentaux
  de Ruby. Ne devrait pas prendre plus de vingt minutes de votre temps !

[Passer d’un autre langage à Ruby](/fr/documentation/ruby-from-other-languages/)
: Vous êtes familier avec un autre langage de programmation et Ruby vous
  intéresse ? Que vous programmiez actuellement en C, C++, Java, Perl,
  PHP ou Python… cet article vous sera utile !

[Je veux apprendre Ruby][2]
: Ce site liste les principales ressources pour apprendre Ruby.

[ROR French Casts](http://www.youtube.com/user/Berlimioz)
: Une série de casts en français sur Ruby et Rails.

#### Ressources en anglais

[Try Ruby!][3]
: Un tutoriel interactif qui vous permet d’essayer Ruby dans votre
  navigateur. Ce tutoriel de 15 minutes est destiné aux débutants qui
  veulent avoir un premier aperçu du langage.

[Ruby Koans][4]
: Les Koans vous emmènent le long du chemin de l’illumination pour vous
  apprendre Ruby. Le but est de vous apprendre le langage Ruby, sa
  syntaxe, sa structure et ses fonctions et bibliothèques courantes. Il
  vous enseignera aussi quelques points « culturels » (bonnes pratiques,
  état d’esprit du programmeur Ruby…)

[Why’s (Poignant) Guide to Ruby][5]
: Ce livre, à la frontière entre tutoriel, roman et œuvre d’art, vous
  propose une manière non conventionnelle mais intéressante d’apprendre
  Ruby au travers d’histoires, de mots d’esprit et de dessins.
  Initialement créé par why the lucky stiff, ce guide est devenu un
  classique.

[*Learn to Program*][6]
: Un magnifique petit tutorial destiné aux débutants, par Chris Pine. Si
  vous n’avez aucunes notions de programmation, commencez par là. Une
  traduction française est [disponible en PDF][7]

[*Ruby Essentials*][10]
: Un *ebook* gratuit qui se veut synthétique et facile d’accès.

[*Ruby &amp; Ruby on Rails programming tutorials*][11]
: Site web proposant une quantité certaines de tutoriaux tant sur Ruby
  que sur Rails.

### Manuels

[*Programming Ruby*][12]
: Le manuel fondateur sur Ruby en langue anglaise : il s’agit ici de la
  première version du [livre le plus populaire sur Ruby][13],
  par les *Pragmatic Programmers*.

[*The Ruby Programming Wikibook*][15]
: Un manuel gratuit destiné aux débutants et aux programmeurs de niveaux
  intermédiaire. Propose également une référence du langage.

### Documentation de référence

[*Ruby Core Reference*][16]
: Extraite directement des sources grâce à [RDoc][17], cette référence
  décrit toutes les classes et modules de bases (comme `String`,
  `Array`, `Symbol`, etc.

[*Ruby Standard Library Reference*][18]
: Également extraite du code source par RDoc, cette seconde référence
  documente la bibliothèque standard.

[*Ruby C API Reference*][extensions]
: La documentation de l’API C de Ruby.
  Très utile si vous comptez écrire des extensions C.

[rubydoc.info][20]
: Documentation auto-générée pour un grand nombre de bibliothèques Ruby.

[Ruby & Rails Searchable API Docs][21]
: Documentation sur les API Ruby et Ruby On Rails, proposant un système
  de recherche poussé.

### Lectures additionnelles

[Ruby-Doc.org][22] maintient à jour une liste presque exhaustive de la
documentation disponible en anglais. Il existe également de nombreux
ouvrages à propos de Ruby : [une liste de livres en français][24]
(incomplète). Par ailleurs, si vous avez des questions à poser sur Ruby,
la [liste de diffusion](/en/community/mailing-lists/) est un bon endroit
à explorer.



[2]: http://jeveuxapprendreruby.fr/
[3]: https://try.ruby-lang.org/
[4]: https://rubykoans.com/
[5]: https://poignant.guide
[6]: http://pine.fm/LearnToProgram/
[7]: http://www.ruby-doc.org/docs/ApprendreProgrammer/Apprendre_%E0_Programmer.pdf
[10]: http://www.techotopia.com/index.php/Ruby_Essentials
[11]: http://www.meshplex.org/wiki/Ruby/Ruby_on_Rails_programming_tutorials
[12]: http://www.ruby-doc.org/docs/ProgrammingRuby/
[13]: http://pragmaticprogrammer.com/titles/ruby/index.html
[15]: http://en.wikibooks.org/wiki/Ruby_programming_language
[16]: http://www.ruby-doc.org/core
[17]: https://ruby.github.io/rdoc/
[18]: http://www.ruby-doc.org/stdlib
[extensions]: https://docs.ruby-lang.org/en/master/extension_rdoc.html
[20]: http://rubydoc.info/
[21]: http://rubydocs.org/
[22]: http://ruby-doc.org
[24]: http://rubyfrance.org/liens/livres/
