---
layout: page
title: "PerlからRubyへ"
lang: ja
---

Perlはすばらしい言語です。ドキュメントもすばらしいし、Perlコミュニティもすばらしいコミュニティです。
最初から洗練されたオブジェクト指向の機能が組み込まれた言語を求めているPerl使いにとって、
Rubyは答えになるかもしれません。

### Perlとの類似

Perlと同様に、Rubyは…

* CPANのような、[RubyGems][1]という名前のパッケージ管理システムがあります。
* 正規表現エンジンが組み込まれています。
* たくさんの便利な組み込みライブラリがあります。
* 括弧はしばしば省略されます。
* 文字列は基本的に同じように使えます。
* 一般的な区切り文字や正規表現構文はPerlと同様です。
  `%q{this}`(一重引用符)や`%Q{this}`(二重引用符)、
  `%w{this for a single-quoted list of words}`のようになります。
  区切り文字は、望むなら`%Q|can|` `%Q(use)` `%Q^other^`のようにもできます。
* 二重引用符の文字列中に、`"looks #{like} this"` のような形で式展開すること
  ができます(`#{}`の中に任意のRubyコードを置くことができます)。
* シェルコマンドを実行するには`` `backticks` ``のようにします。
* rdocという組み込みのドキュメントツールがあります。

### Perlとの違い

Perlと違って、Rubyは…

* Perlのようなコンテキスト依存のルールはありません。
* 変数はオブジェクトそのものではありません。常にオブジェクトへの参照です。
* 変数名の最初の1文字に`$`や`@`などが使われることがあります。
  これらは型ではなく、スコープを示します
  (`$`はグローバル変数、`@`はインスタンス変数、`@@`はクラス変数
  であることを示します)。
* 配列リテラルは括弧ではなく角括弧を使います。
* リストの中にリストを入れると、それはフラットな一つのリスト
  になるわけではありません。代わりに配列の配列が得られます。
* `sub`の代わりに`def`を使います。
* 行の終わりにセミコロンは必要ありません。
  関数定義やクラス定義、case文の終わりを示すには、
  `end`キーワードを使います。
* オブジェクトは強く型付けされています。
  他の型に変換するには、`foo.to_i`や`foo.to_s`などの変換メソッドを
  呼び出す必要があります。
* `eq`、`ne`、`lt`、`gt`、`ge`、`le`はありません。
* ダイアモンド演算子(`<>`)はありません。
  `IO.some_method`を代わりに使います。
* `=>`はハッシュリテラルにだけ使われます。
* `undef`はありません。Rubyには`nil`があります。
  `nil`は(Rubyにおける他の値と同様に)オブジェクトで、
  未定義の変数とは異なります。
  真偽値のように扱った場合、`nil`は`false`と評価されます。
* 真か偽かの判定では、`false`と`nil`のみが偽と評価されます。
  それ以外の値(`0`、`0.0`、`"0"`など)はすべて真と評価されます。
* [PerlMonks][2]はありませんが、ruby-listメーリングリストなどの質問場所があります。



[1]: http://guides.rubygems.org
[2]: http://www.perlmonks.org/
