---
layout: page
title: "Documentação"
lang: pt
---

Aqui você encontrará vários endereços para manuais, tutoriais e
referências que serão úteis quando você sentir que gosta de programar
em Ruby.
{: .summary}

### Instalando o Ruby

A menos que você queira testar o Ruby no seu navegador (veja os links abaixo),
você precisará ter o Ruby instalado no seu computador. Você pode verificar se
o Ruby já está instalado abrindo o terminal e digitando

{% highlight sh %}
ruby -v
{% endhighlight %}

Isso deve imprimir algumas informações sobre a versão do Ruby instalada. Se não
imprimir, consulte a [página de instalação](installation/) para conhecer as
diversas maneiras de obter o Ruby.

### Primeiros Passos

[Try Ruby!][1]
: Um tutorial interativo que permite que você teste o Ruby diretamente no
  seu browser. Este tutorial de 15 minutos é destinado para iniciantes que
  queiram ter uma ideia de como é a linguagem.

[Ruby Koans][2]
: Os Koans te conduzem pelo caminho da iluminação de modo a aprender Ruby.
  O objetivo é aprender a linguagem, sintaxe, estrutura algumas funções e
  bibliotecas comuns do Ruby. Também ensinamos cultura.

[O Guia (Comovente) de Ruby do Why][5]
: UM livro inconveniente, porém interessante, que te ensinará Ruby através
  de histórias, humor e quadrinhos. Originalmente criado por *why the lucky
  stiff*, esse guia permanece um clássico para os aprendizes de Ruby.

[Ruby em Vinte Minutos](/pt/documentation/quickstart/)
: Um bom tutorial que cobre o básico de Ruby. Desde o início ao fim, não
  deverá levar mais de vinte minutos.

[Ruby a Partir de Outras Linguagens](/pt/documentation/ruby-from-other-languages/)
: Chegou ao Ruby por outra linguagem? Quer seja C, C++, Java, Perl,
  PHP ou Python, este artigo é para você!

[Ruby Essentials][7]
: Ruby Essentials é um livro digital gratuito projetado para prover um
  guia conciso e fácil de seguir para o aprendiz de Ruby.

[Aprenda a Programar][8]
: Pequeno e maravilhoso tutorial por Chris Pine para novatos em
  programação. Se não sabe programar, comece aqui.

[Learn Ruby the Hard Way][38]
: Um ótimo conjunto de exercícios com explicações que o conduzem do básico
  do Ruby por todo o caminho até a OOP e o desenvolvimento para a web.

### Manuais

[Programming Ruby][9]
: Trabalho seminal de Ruby em inglês, a primeira edição do [Pragmatic
  Programmers’ book][10] está disponível gratuitamente online.

[The Ruby Programming Wikibook][12]
: Manual online gratuito, com conteúdo para iniciantes e intermediário,
  além de uma referência completa para a linguagem.

### Documentação de Referência

[Ruby Core Reference][13]
: Vindo diretamente do código-fonte através de [RDoc][14], esta
  referência documenta todas as classes e módulos do _core_ (como String,
  Array, Symbol, etc…).

[Ruby Standard Library Reference][15]
: Também vindo do código-fonte através do RDoc, esta referência explora
  documentos da biblioteca padrão.

[RubyDoc.info][16]
: O website essencial para documentação de referência sobre as Gems do Ruby e
  projetos Ruby hospedados no GitHub.

[Ruby & Rails Searchable API Docs][17]
: Documentação do Ruby e do Rails com busca inteligente.

[APIdock][18]
: Documentação do Ruby, Rails e RSpec com notas de usuários.

### Editores e IDEs

Para programar em Ruby, você pode usar o editor padrão do seu sistema
operacional. A propósito, para programar mais eficazmente, pode valer
a pena escolher um editor de código fonte com suporte básico a Ruby
(por exemplo, que colore a sintaxe, lista os arquivos, etc.) ou um
ambiente integrado de desenvolvimento com funcionalidades avançadas
(como completar o código, refatorar, suporte a testes, etc.).

Segue aqui uma lista das ferramentas populares utilizadas por
programadores Ruby:

* Ferramentas para Linux e outras plataformas:
  * [Aptana Studio][19]
  * [Emacs][20] com [Ruby mode][21] e [Rsense][22]
  * [Geany][23]
  * [gedit][24]
  * [Vim][25] com o plugin [vim-ruby][26] e [Rsense][22]
  * [RubyMine][27]
  * [SciTe][28]
  * [NetBeans][36]
  * [Sublime Text][37]
  * [Atom][atom]
  * [Visual Studio Code][vscode]

* No Windows:
  * [Notepad++][29]

* No macOS:
  * [TextMate][32]
  * [BBEdit][33]
  * [Dash][39] (navegador de documentação)

### Leitura Complementar

[Ruby-Doc.org][34] mantém uma lista completa de fontes de documentação
em inglês. Se você tiver
perguntas sobre Ruby, a [lista de e-mails](/pt/community/mailing-lists/)
é um ótimo lugar para começar.



[1]: https://try.ruby-lang.org/
[2]: https://rubykoans.com/
[5]: http://why.carlosbrando.com/
[7]: http://www.techotopia.com/index.php/Ruby_Essentials
[8]: http://aprendaaprogramar.rubyonrails.com.br/
[9]: http://www.ruby-doc.org/docs/ProgrammingRuby/
[10]: http://pragmaticprogrammer.com/titles/ruby/index.html
[12]: http://en.wikibooks.org/wiki/Ruby_programming_language
[13]: http://www.ruby-doc.org/core
[14]: https://ruby.github.io/rdoc/
[15]: http://www.ruby-doc.org/stdlib
[16]: http://www.rubydoc.info/
[17]: http://rubydocs.org/
[18]: http://apidock.com/
[19]: http://www.aptana.com/
[20]: http://www.gnu.org/software/emacs/
[21]: http://www.emacswiki.org/emacs/RubyMode
[22]: http://rsense.github.io/
[23]: http://www.geany.org/
[24]: http://projects.gnome.org/gedit/screenshots.html
[25]: http://www.vim.org/
[26]: https://github.com/vim-ruby/vim-ruby
[27]: http://www.jetbrains.com/ruby/
[28]: http://www.scintilla.org/SciTE.html
[29]: http://notepad-plus-plus.org/
[32]: http://macromates.com/
[33]: https://www.barebones.com/products/bbedit/
[34]: http://ruby-doc.org
[36]: https://netbeans.org/
[37]: http://www.sublimetext.com/
[38]: https://learncodethehardway.org/ruby/
[39]: http://kapeli.com/dash
[atom]: https://atom.io/
[vscode]: https://code.visualstudio.com/
