---
layout: page
title: "Ruby Core"
lang: ru
---

Сейчас отличное время, чтобы участвовать в разработке Ruby. С повышенным
вниманием, которое получил Ruby в последние несколько лет, в сообществе
возрастает потребность в талантах для улучшения Ruby самого по себе и
его документации. Итак, где вам начать?
{: .summary}

Темы, относящиеся к разработке Ruby, находятся здесь:

* [Использование Subversion для прослеживания процесса разработки Ruby](#following-ruby)
* [Как использовать Git для работы с главным репозиторием Ruby](#git-ruby)
* [Улучшаем Ruby, патч за патчем](#patching-ruby)
* [Правила для разработчиков ядра](#coding-standards)

### Использование Subversion для прослеживания разработки Ruby
{: #following-ruby}

Анонимно получить последние исходники Ruby можно из [Subversion][1]
репозитория. В командной строке наберите:

{% highlight sh %}
$ svn co https://svn.ruby-lang.org/repos/ruby/trunk ruby
{% endhighlight %}

Папка `ruby` теперь будет содержать последние исходники
development-версии Ruby (ruby-trunk). Патчи, применяемые к ruby-trunk
так же портируются в стабильные
{{ site.svn.stable.version }}, {{ site.svn.previous.version }},
и {{ site.svn.old.version }} ветки (смотри ниже).

Если вы хотите следить за патчингом Ruby {{ site.svn.stable.version }}, вы можете переключится
на `{{ site.svn.stable.branch }}` ветку:

{% highlight sh %}
$ svn co https://svn.ruby-lang.org/repos/ruby/branches/{{ site.svn.stable.branch }}
{% endhighlight %}

То же самое для Ruby {{ site.svn.previous.version }}:

{% highlight sh %}
$ svn co https://svn.ruby-lang.org/repos/ruby/branches/{{ site.svn.previous.branch }}
{% endhighlight %}

Эти команды скачают соответствующее дерево разработки в папку
`{{ site.svn.stable.branch }}` или `{{ site.svn.previous.branch }}`.
Разработчики, работающие над поддерживаемыми ветками,
ожидают применения их изменений в ruby-trunk, так что зачастую обе ветки
очень похожи, за исключением изменений, сделанных Matz'ем и Nobu в языке
в целом.

Если вы предпочитаете, вы можете открыть [Subversion репозиторий Ruby в
браузере][2].

Для информации по поводу Subversion, пожалуйста смотрите [the Subversion FAQ][3]
и книгу [the Subversion book][4]. Еще вы можете найти полезной для начала
изучения Subversion книгу [Pragmatic Version Control with Subversion][5].

### Как использовать Git для работы с главным репозиторием Ruby
{: #git-ruby}

Для тех, кто предпочитает использовать [Git][6] вместо Subversion,
инструкции могут найдены в [зеркале репозитория на GitHub][7]. Их две,
одна для [тех, кто имеет право коммитить][8] и другая – для [всех
остальных][9].

### Улучшаем Ruby, патч за патчем
{: #patching-ruby}

Разработчики ядра поддерживают [специальный форум][10] для приема патчей
и сообщений о багах для Matz'а и команды. Эти сообщения также попадают в
[Ruby-Core mailing list][mailing-lists] для обсуждения,
так что вы можете быть уверены, что ваше сообщение не останется без
внимания. Вы также можете отправлять свои патчи прямиком в почтовую рассылку.
В любом случае, вам предлагается принять участие в последующей
дискуссии.

Пожалуйста, посмотрите [Patch Writer's Guide][11]. Вы найдете там
некоторые подсказки и приемы непосредственно от Matz'а о том, как
сделать так, чтоб ваши патчи приняли.

Суммируя шаги создания патча:

1.  Скачайте копию исходного кода Ruby из Subversion. Обычно, патчи
    которые исправляют баги или добавляют новые конструкции, должны быть
    применены к trunk ветке исходного кода Ruby. Даже если вы хотите
    добавить конструкцию к {{ site.svn.previous.version }}, это должно быть сначала проверено в
    trunk.

        $ svn co https://svn.ruby-lang.org/repos/ruby/trunk ruby

    Если вы исправляете баг, который специфичен для одной из
    поддерживаемых веток, выберите эту ветку, например, `{{ site.svn.previous.version }}`.

        $ svn co https://svn.ruby-lang.org/repos/ruby/branches/{{ site.svn.previous.branch }}

2.  Добавьте ваши улучшения в код.

3.  Создайте патч.

        $ svn diff > ruby-changes.patch

4.  Создайте тикет в [специальном форуме][10] или вышлите свой патч в
    рассылку [Ruby-Core mailing list][mailing-lists]
    с записью в Changelog, описывающую ваши изменения.

5.  Если с патчем не возникнет никаких проблем, программисты имеющие
    права на добавление кода в репозитории, добавят ваш патч.

**Пожалуйста, обратите внимание:** что патчи должны быть предложены как
[унифицированный дифф][12]. Для большей информации о том как применяют
патчи, пожалуйста читайте [the diffutils reference][13].

Обсуждение разработки Ruby происходит в почтовой рассылке
[Ruby-Core mailing list][mailing-lists]. Таким образом, если вы размышляете
о том, будет ли ваш патч полезен, или вы хотите начать обсуждение о
будущем Ruby, не медлите начать общение. Учтите также, что обсуждения не
по теме не приветствуются в данной рассылке. Уровень шума должен быть
низок, темы должны быть точными, продуманными и хорошо написанными. Так
как рассылка приходит создателю Ruby, давайте будем вежливы.

Держите в уме, несмотря на то, что разработчики ядра Ruby большинством
своим живут в Японии и многие из них владеют английским очень хорошо,
присутствует значительная разница во времени. Разработчики ядра также
имеют большое количество обсуждаемых на японском языке вещей, которые
происходят параллельно с задачами, обсуждаемыми на английском. Будьте
терпеливы и, если ваша проблема не решена, будьте настойчивы в следующей
попытке через несколько дней.

### Правила для разработчиков ядра
{: #coding-standards}

Обычно, разработчики Ruby должны быть знакомы с исходным кодом и стилем
разработки, используемым командой. Но на всякий случай, следующие
правила должны уважаться, когда вы что-то изменяете:

* Все добавления должны быть описаны в `ChangeLog` файле, следуя [GNU
  конвенции][14]. (Многие разработчики ядра используют режим Emacs'а
  `add-log`, который может быть включен командой `C-x 4 a`.)
* Даты изменений должны быть представлены в формате Japan Standard Time
  (UTC+9).
* Буллиты из вашего ChangeLog должны также быть представлены в
  комментарии к вашему коммиту в Subversion. Сообщение будет
  автоматически выслано в рассылку Ruby-CVS после вашего коммита.
* Прототипы функций используются в исходном коде Ruby и его расширениях.
* Пожалуйста, не используйте C++ комментарии (`//`), разработчики ядра
  предпочитают стиль многострочных комментариев C (`/* .. */`).

Также, для большей информации читайте [Ruby's issue tracker][10]



[mailing-lists]: /ru/community/mailing-lists/
[1]: http://subversion.apache.org/
[2]: https://svn.ruby-lang.org/cgi-bin/viewvc.cgi/
[3]: http://subversion.apache.org/faq.html
[4]: http://svnbook.org
[5]: http://www.pragmaticprogrammer.com/titles/svn/
[6]: http://git-scm.com/
[7]: https://github.com/ruby/ruby
[8]: https://github.com/shyouhei/ruby/wiki/committerhowto
[9]: https://github.com/shyouhei/ruby/wiki/noncommitterhowto
[10]: https://bugs.ruby-lang.org/
[11]: https://blade.ruby-lang.org/ruby-core/25139
[12]: http://www.gnu.org/software/diffutils/manual/html_node/Unified-Format.html
[13]: http://www.gnu.org/software/diffutils/manual/html_node/Merging-with-patch.html#Merging%20with%20patch
[14]: http://www.gnu.org/prep/standards/standards.html#Change-Logs
