---
layout: page
title: "文档"
lang: zh_cn
---

在这里您可以找到手册、教程和参考资料的链接，在你爱上使用 Ruby 编程的时候，它们将帮上大忙。
{: .summary}

### 安装 Ruby

除非只想在浏览器里尝试 Ruby（查看下面的链接），否则你需要在自己的电脑中安装 Ruby。打开终端，输入以下命令检查 Ruby 是否可用：

{% highlight sh %}
ruby -v
{% endhighlight %}

这个命令输出已安装 Ruby 的版本信息。如无显示，请访问[安装页面](installation/)查看安装 Ruby 的各种方法。

### 入门

[Try Ruby！][1]
: 可以在浏览器内体验 Ruby 的交互式教程。这个 15 分钟的教程主要面向想尝试这门语言的初学者。

[Ruby Koans][2]
: Ruby Koans 可以指引你走过学习 Ruby 的启蒙之路。在这里可以学到 Ruby 语言的语法、结构、常用函数和库。当然，还有 Ruby 文化。

[Why’s (Poignant) Guide to Ruby][5]
: 一本不同寻常但是非常有趣的书，通过故事、幽默和漫画教你 Ruby。由 *why the luckystiff* 创作, 本书始终是学习 Ruby 的经典之作。

[Ruby 20 分钟入门](/zh_cn/documentation/quickstart/)
: 这是个不错的教程，介绍 Ruby 基础知识。从开始到结束不超过 20 分钟。


[从其它语言转到 Ruby](/zh_cn/documentation/ruby-from-other-languages/)
: 你是从其他语言转到 Ruby 的吗？不管是 C、C++、Java、Perl、PHP，还是 Python，这里都有介绍！

[Ruby Essentials][7]
: Ruby Essentials 是一本免费在线书籍，旨在提供简洁易懂的 Ruby 学习指南。

[Learn to Program][8]
: 非常棒的小教程，由 Chris Pine 为编程入门者编写。如果你不知道如何编程，从这里开始吧。

[笨方法学 Ruby][38]
: 一系列非常好的练习，附带讲解，可以指导你从基本的 Ruby 知识一路进阶到面向对象编程和 Web 开发。

### 手册

[Programming Ruby][9]
: 最有影响的 Ruby 英文教材，[Pragmatic Programmers 出版的第一版][10]可以在网上免费阅读。

[Ruby 编程百科全书][12]
: 免费的在线语言参考资料，内容从 Ruby 初级到中级。

### 参考文档

[Ruby 内核参考][13]
: 直接从源代码中的 [RDoc][14] 提取，这个文档包含了所有的核心类和模块（比如 String，Array，Symbol 等）。

[Ruby 标准库参考][15]
: 同样从源代码中提取，这是标准库的参考。

[RubyDoc.info][16]
:  关于 Ruby gems 和 GitHub 上托管的 Ruby 项目的参考文档的一站式站点。

[Ruby 和 Rails 可检索 API 文档][17]
: 可以智能搜索的 Rails 和 Ruby 文档。

[APIdock][18]
: 带有用户评注的 Ruby、Rails 和 RSpec 文档。

[RubyAPI.org][rubyapi-org]
: 简单搜索浏览 Ruby 类、模块和方法。

### 编辑器和集成开发环境

您可以用操作系统中默认的编辑器来进行 Ruby 编程。不过，为了更高效地编程，建议选择一个支持 Ruby 基本特性（比如句法高亮，文件浏览）的代码编辑器，或者有高级特性（比如代码补全，重构，测试支持等）的集成开发环境。

下面是 Ruby 程序员常用的工具列表：

* Linux 和跨平台工具:
  * [Aptana Studio][19]
  * [Emacs][20]，安装 [Ruby 模式][21]和 [Rsense][22]
  * [Geany][23]
  * [gedit][24]
  * [Vim][25]，安装 [vim-ruby][26] 插件和 [Rsense][22]
  * [RubyMine][27]
  * [SciTe][28]
  * [NetBeans][36]
  * [Sublime Text][37]
  * [Atom][atom]
  * [Visual Studio Code][vscode]

* Windows:
  * [Notepad++][29]

* macOS:
  * [TextMate][32]
  * [BBEdit][33]
  * [Dash][39]（文档浏览器）

### 扩展阅读

[Ruby-Doc.org][34] 网站中有完整的 Ruby 英文文档。如果有 Ruby 相关的问题，可以参加[邮件列表](/zh_cn/community/mailing-lists/)中的讨论。



[1]: https://try.ruby-lang.org/
[2]: https://rubykoans.com/
[5]: https://poignant.guide
[7]: http://www.techotopia.com/index.php/Ruby_Essentials
[8]: http://pine.fm/LearnToProgram/
[9]: http://www.ruby-doc.org/docs/ProgrammingRuby/
[10]: http://pragmaticprogrammer.com/titles/ruby/index.html
[12]: http://en.wikibooks.org/wiki/Ruby_programming_language
[13]: http://www.ruby-doc.org/core
[14]: https://ruby.github.io/rdoc/
[15]: http://www.ruby-doc.org/stdlib
[16]: http://www.rubydoc.info/
[17]: http://rubydocs.org/
[18]: http://apidock.com/
[rubyapi-org]: https://rubyapi.org/
[19]: http://www.aptana.com/
[20]: http://www.gnu.org/software/emacs/
[21]: http://www.emacswiki.org/emacs/RubyMode
[22]: http://rsense.github.io/
[23]: http://www.geany.org/
[24]: http://projects.gnome.org/gedit/screenshots.html
[25]: http://www.vim.org/
[26]: https://github.com/vim-ruby/vim-ruby
[27]: http://www.jetbrains.com/ruby/
[28]: http://www.scintilla.org/SciTE.html
[29]: http://notepad-plus-plus.org/
[32]: http://macromates.com/
[33]: https://www.barebones.com/products/bbedit/
[34]: http://ruby-doc.org
[36]: https://netbeans.org/
[37]: http://www.sublimetext.com/
[38]: https://learncodethehardway.org/ruby/
[39]: http://kapeli.com/dash
[atom]: https://atom.io/
[vscode]: https://code.visualstudio.com/
