---
layout: news_post
title: "Desbordamiento de Pila en la interpretacion de URIs en YAML (CVE-2014-2525)"
author: "hone and zzak"
translator: "David Padilla"
date: 2014-03-29 01:49:25 +0000
tags: security
lang: es
---

Hay un desbordamiento en la interpretación de URIs de YAML en Ruby.
Se le ha asignado el identificador [CVE-2014-2525](http://www.ocert.org/advisories/ocert-2014-003.html)
a esta vulnerabilidad.

## Detalles

En cualquier momento en el que un YAML con etiquetas es interpretado, una
cadena especialmente preparada puede causar un desbordamiento de pila lo que
puede llevar a ejecución de código arbitrario.

Por ejemplo:

{% highlight ruby %}
YAML.load <codigo_de_fuente_desconocida>
{% endhighlight %}

## Versiones Afectadas

Las versiones de Ruby 1.9.3-p0 y en delante incluyen psych como el interprete
YAML por defecto. Cualquier versión de psych que ha sido compilada con
libyaml `<= 0.1.5` se considera afectada.

Estas versiones de Ruby contienen una versión afectada de libyaml:

* Ruby 2.0.0-p451 y anteriores,
* Ruby 2.1.0 y Ruby 2.1.1.

Puedes verificar la versión de libyaml usada ejecutando:

{% highlight sh %}
$ ruby -rpsych -e 'p Psych.libyaml_version'
[0, 1, 5]
{% endhighlight %}

## Soluciones

Se recomienda a los usuarios que instalaron libyaml en el sistema que lo actualicen
a `0.1.6`. Al recompilar Ruby se hará referencia a la versión actualizada de libyaml:

{% highlight sh %}
$ ./configure --with-yaml-dir=/path/to/libyaml
{% endhighlight %}

Se recomienda a los usuarios sin una version libyaml en su sistema y que confían en la versión
incluida en Ruby que actualicen psych a la versión `2.0.5` la cual contiene
la versión `0.1.6` de libyaml:

{% highlight sh %}
$ gem install psych
{% endhighlight %}

o actualiza tu versión de Ruby a 2.0.0-p481, 2.1.2 o más nueva.

## Historia

* Publicado originalmente 2014-03-29 01:49:25 UTC
* Actualizado 2014-03-29 09:37:00 UTC
* Actualizado 2014-05-09 03:00:00 UTC
